@@ -3,12 +3,12 @@
  * varchar.c
  *	  Functions for the built-in types char(n) and varchar(n).
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/varchar.c,v 1.95.2.1 2006/05/21 20:07:11 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/varchar.c,v 1.102 2003/08/04 04:03:10 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -75,7 +75,10 @@ bpcharin(PG_FUNCTION_ARGS)
 	int			i;
 	int			charlen;		/* number of charcters in the input string */
 
+	/* verify encoding */
 	len = strlen(s);
+	pg_verifymbstr(s, len, false);
+
 	charlen = pg_mbstrlen(s);
 
 	/* If typmod is -1 (or invalid), use the actual string length */
@@ -97,8 +100,10 @@ bpcharin(PG_FUNCTION_ARGS)
 		if (strspn(s + mbmaxlen, " ") == len - mbmaxlen)
 			len = mbmaxlen;
 		else
-			elog(ERROR, "value too long for type character(%d)",
-				 (int) maxlen);
+			ereport(ERROR,
+					(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
+					 errmsg("value too long for type character(%d)",
+							(int) maxlen)));
 
 		/*
 		 * XXX: at this point, maxlen is the necessary byte length, not
@@ -125,10 +130,6 @@ bpcharin(PG_FUNCTION_ARGS)
 	for (; i < maxlen; i++)
 		*r++ = ' ';
 
-#ifdef CYR_RECODE
-	convertstr(VARDATA(result), len, 0);
-#endif
-
 	PG_RETURN_BPCHAR_P(result);
 }
 
@@ -149,13 +150,29 @@ bpcharout(PG_FUNCTION_ARGS)
 	memcpy(result, VARDATA(s), len);
 	result[len] = '\0';
 
-#ifdef CYR_RECODE
-	convertstr(result, len, 1);
-#endif
-
 	PG_RETURN_CSTRING(result);
 }
 
+/*
+ *		bpcharrecv			- converts external binary format to bpchar
+ */
+Datum
+bpcharrecv(PG_FUNCTION_ARGS)
+{
+	/* Exactly the same as textrecv, so share code */
+	return textrecv(fcinfo);
+}
+
+/*
+ *		bpcharsend			- converts bpchar to binary format
+ */
+Datum
+bpcharsend(PG_FUNCTION_ARGS)
+{
+	/* Exactly the same as textsend, so share code */
+	return textsend(fcinfo);
+}
+
 
 /*
  * Converts a CHARACTER type to the specified size.
@@ -165,7 +182,7 @@ bpcharout(PG_FUNCTION_ARGS)
  *
  * Truncation rules: for an explicit cast, silently truncate to the given
  * length; for an implicit cast, raise error unless extra characters are
- * all spaces.  (This is sort-of per SQL: the spec would actually have us
+ * all spaces.	(This is sort-of per SQL: the spec would actually have us
  * raise a "completion condition" for the explicit cast case, but Postgres
  * hasn't got such a concept.)
  */
@@ -203,8 +220,10 @@ bpchar(PG_FUNCTION_ARGS)
 		{
 			for (i = maxmblen - VARHDRSZ; i < len - VARHDRSZ; i++)
 				if (*(VARDATA(source) + i) != ' ')
-					elog(ERROR, "value too long for type character(%d)",
-						 maxlen - VARHDRSZ);
+					ereport(ERROR,
+							(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
+						  errmsg("value too long for type character(%d)",
+								 maxlen - VARHDRSZ)));
 		}
 
 		len = maxmblen;
@@ -316,7 +335,10 @@ name_bpchar(PG_FUNCTION_ARGS)
 
 
 /*****************************************************************************
- *	 varchar - varchar()													 *
+ *	 varchar - varchar(n)
+ *
+ * Note: varchar piggybacks on type text for most operations, and so has no
+ * C-coded functions except for I/O and typmod checking.
  *****************************************************************************/
 
 /*
@@ -342,7 +364,10 @@ varcharin(PG_FUNCTION_ARGS)
 	size_t		len,
 				maxlen;
 
+	/* verify encoding */
 	len = strlen(s);
+	pg_verifymbstr(s, len, false);
+
 	maxlen = atttypmod - VARHDRSZ;
 
 	if (atttypmod >= (int32) VARHDRSZ && len > maxlen)
@@ -353,18 +378,16 @@ varcharin(PG_FUNCTION_ARGS)
 		if (strspn(s + mbmaxlen, " ") == len - mbmaxlen)
 			len = mbmaxlen;
 		else
-			elog(ERROR, "value too long for type character varying(%d)",
-				 (int) maxlen);
+			ereport(ERROR,
+					(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
+				  errmsg("value too long for type character varying(%d)",
+						 (int) maxlen)));
 	}
 
 	result = palloc(len + VARHDRSZ);
 	VARATT_SIZEP(result) = len + VARHDRSZ;
 	memcpy(VARDATA(result), s, len);
 
-#ifdef CYR_RECODE
-	convertstr(VARDATA(result), len, 0);
-#endif
-
 	PG_RETURN_VARCHAR_P(result);
 }
 
@@ -385,13 +408,29 @@ varcharout(PG_FUNCTION_ARGS)
 	memcpy(result, VARDATA(s), len);
 	result[len] = '\0';
 
-#ifdef CYR_RECODE
-	convertstr(result, len, 1);
-#endif
-
 	PG_RETURN_CSTRING(result);
 }
 
+/*
+ *		varcharrecv			- converts external binary format to varchar
+ */
+Datum
+varcharrecv(PG_FUNCTION_ARGS)
+{
+	/* Exactly the same as textrecv, so share code */
+	return textrecv(fcinfo);
+}
+
+/*
+ *		varcharsend			- converts varchar to binary format
+ */
+Datum
+varcharsend(PG_FUNCTION_ARGS)
+{
+	/* Exactly the same as textsend, so share code */
+	return textsend(fcinfo);
+}
+
 
 /*
  * Converts a VARCHAR type to the specified size.
@@ -401,7 +440,7 @@ varcharout(PG_FUNCTION_ARGS)
  *
  * Truncation rules: for an explicit cast, silently truncate to the given
  * length; for an implicit cast, raise error unless extra characters are
- * all spaces.  (This is sort-of per SQL: the spec would actually have us
+ * all spaces.	(This is sort-of per SQL: the spec would actually have us
  * raise a "completion condition" for the explicit cast case, but Postgres
  * hasn't got such a concept.)
  */
@@ -431,8 +470,10 @@ varchar(PG_FUNCTION_ARGS)
 	{
 		for (i = maxmblen; i < len - VARHDRSZ; i++)
 			if (*(VARDATA(source) + i) != ' ')
-				elog(ERROR, "value too long for type character varying(%d)",
-					 maxlen - VARHDRSZ);
+				ereport(ERROR,
+						(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
+				  errmsg("value too long for type character varying(%d)",
+						 maxlen - VARHDRSZ)));
 	}
 
 	len = maxmblen + VARHDRSZ;
@@ -647,7 +688,9 @@ bpcharcmp(PG_FUNCTION_ARGS)
 
 /*
  * bpchar needs a specialized hash function because we want to ignore
- * trailing blanks in comparisons.	(varchar can use plain hashvarlena.)
+ * trailing blanks in comparisons.
+ *
+ * XXX is there any need for locale-specific behavior here?
  */
 Datum
 hashbpchar(PG_FUNCTION_ARGS)
@@ -667,187 +710,3 @@ hashbpchar(PG_FUNCTION_ARGS)
 
 	return result;
 }
-
-
-/*****************************************************************************
- *	Functions used for varchar
- *****************************************************************************/
-
-Datum
-varcharlen(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg = PG_GETARG_VARCHAR_P(0);
-
-	/* optimization for single byte encoding */
-	if (pg_database_encoding_max_length() <= 1)
-		PG_RETURN_INT32(VARSIZE(arg) - VARHDRSZ);
-
-	PG_RETURN_INT32(
-			  pg_mbstrlen_with_len(VARDATA(arg), VARSIZE(arg) - VARHDRSZ)
-		);
-}
-
-Datum
-varcharoctetlen(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg = PG_GETARG_VARCHAR_P(0);
-
-	PG_RETURN_INT32(VARSIZE(arg) - VARHDRSZ);
-}
-
-
-/*****************************************************************************
- *	Comparison Functions used for varchar
- *
- * Note: btree indexes need these routines not to leak memory; therefore,
- * be careful to free working copies of toasted datums.  Most places don't
- * need to be so careful.
- *****************************************************************************/
-
-Datum
-varchareq(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	bool		result;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	/* fast path for different-length inputs */
-	if (len1 != len2)
-		result = false;
-	else
-		result = (varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2) == 0);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-varcharne(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	bool		result;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	/* fast path for different-length inputs */
-	if (len1 != len2)
-		result = true;
-	else
-		result = (varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2) != 0);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-varcharlt(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	int			cmp;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(cmp < 0);
-}
-
-Datum
-varcharle(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	int			cmp;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(cmp <= 0);
-}
-
-Datum
-varchargt(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	int			cmp;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(cmp > 0);
-}
-
-Datum
-varcharge(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	int			cmp;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_BOOL(cmp >= 0);
-}
-
-Datum
-varcharcmp(PG_FUNCTION_ARGS)
-{
-	VarChar    *arg1 = PG_GETARG_VARCHAR_P(0);
-	VarChar    *arg2 = PG_GETARG_VARCHAR_P(1);
-	int			len1,
-				len2;
-	int			cmp;
-
-	len1 = VARSIZE(arg1) - VARHDRSZ;
-	len2 = VARSIZE(arg2) - VARHDRSZ;
-
-	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
-
-	PG_FREE_IF_COPY(arg1, 0);
-	PG_FREE_IF_COPY(arg2, 1);
-
-	PG_RETURN_INT32(cmp);
-}
