@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright 2000-2002 by PostgreSQL Global Development Group
+ * Copyright (c) 2000-2003, PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql-server/src/bin/psql/describe.c,v 1.71.2.1 2002/12/21 01:07:21 tgl Exp $
+ * $Header: /cvsroot/pgsql-server/src/bin/psql/describe.c,v 1.87 2003/10/26 02:53:45 momjian Exp $
  */
 #include "postgres_fe.h"
 #include "describe.h"
@@ -18,6 +18,16 @@
 
 #include <ctype.h>
 
+#ifdef WIN32
+/*
+ * mbvalidate() is used in function describeOneTableDetails() to make sure
+ * all characters of the cells will be printed to the DOS console in a
+ * correct way
+ */
+#include "mbprint.h"
+#endif
+
+
 #define _(x) gettext((x))
 
 static bool describeOneTableDetails(const char *schemaname,
@@ -129,10 +139,10 @@ describeFunctions(const char *pattern, bool verbose)
 	initPQExpBuffer(&buf);
 
 	printfPQExpBuffer(&buf,
-		 "SELECT CASE WHEN p.proretset THEN 'setof ' ELSE '' END ||\n"
-				  "  pg_catalog.format_type(p.prorettype, NULL) as \"%s\",\n"
-				  "  n.nspname as \"%s\",\n"
-				  "  p.proname as \"%s\",\n"
+			"SELECT CASE WHEN p.proretset THEN 'setof ' ELSE '' END ||\n"
+			  "  pg_catalog.format_type(p.prorettype, NULL) as \"%s\",\n"
+					  "  n.nspname as \"%s\",\n"
+					  "  p.proname as \"%s\",\n"
 				  "  pg_catalog.oidvectortypes(p.proargtypes) as \"%s\"",
 					  _("Result data type"), _("Schema"), _("Name"),
 					  _("Argument data types"));
@@ -539,7 +549,7 @@ objectDescription(const char *pattern)
 
 	appendPQExpBuffer(&buf,
 					  ") AS tt\n"
-					  "  JOIN pg_catalog.pg_description d ON (tt.oid = d.objoid and tt.tableoid = d.classoid and d.objsubid = 0)\n");
+					  "  JOIN pg_catalog.pg_description d ON (tt.oid = d.objoid AND tt.tableoid = d.classoid AND d.objsubid = 0)\n");
 
 	appendPQExpBuffer(&buf, "ORDER BY 1, 2, 3;");
 
@@ -678,7 +688,7 @@ describeOneTableDetails(const char *schemaname,
 	if (PQntuples(res) == 0)
 	{
 		if (!QUIET())
-			fprintf(stderr, _("Did not find any relation with oid %s.\n"),
+			fprintf(stderr, _("Did not find any relation with OID %s.\n"),
 					oid);
 		goto error_return;
 	}
@@ -711,12 +721,11 @@ describeOneTableDetails(const char *schemaname,
 	headers[cols] = NULL;
 
 	/* Get column info (index requires additional checks) */
-	if (tableinfo.relkind == 'i')
-		printfPQExpBuffer(&buf, "SELECT\n  CASE i.indproc WHEN ('-'::pg_catalog.regproc) THEN a.attname\n  ELSE SUBSTR(pg_catalog.pg_get_indexdef(attrelid),\n  POSITION('(' in pg_catalog.pg_get_indexdef(attrelid)))\n  END,");
-	else
-		printfPQExpBuffer(&buf, "SELECT a.attname,");
-	appendPQExpBuffer(&buf, "\n  pg_catalog.format_type(a.atttypid, a.atttypmod),\n"
-					  "  a.attnotnull, a.atthasdef, a.attnum");
+	printfPQExpBuffer(&buf, "SELECT a.attname,");
+	appendPQExpBuffer(&buf, "\n  pg_catalog.format_type(a.atttypid, a.atttypmod),"
+	"\n  (SELECT substring(d.adsrc for 128) FROM pg_catalog.pg_attrdef d"
+					  "\n   WHERE d.adrelid = a.attrelid AND d.adnum = a.attnum AND a.atthasdef),"
+					  "\n  a.attnotnull, a.attnum");
 	if (verbose)
 		appendPQExpBuffer(&buf, ", pg_catalog.col_description(a.attrelid, a.attnum)");
 	appendPQExpBuffer(&buf, "\nFROM pg_catalog.pg_attribute a");
@@ -737,7 +746,7 @@ describeOneTableDetails(const char *schemaname,
 	{
 		PGresult   *result;
 
-		printfPQExpBuffer(&buf, "SELECT pg_catalog.pg_get_viewdef('%s'::pg_catalog.oid)", oid);
+		printfPQExpBuffer(&buf, "SELECT pg_catalog.pg_get_viewdef('%s'::pg_catalog.oid, true)", oid);
 		result = PSQLexec(buf.data, false);
 		if (!result)
 			goto error_return;
@@ -755,47 +764,52 @@ describeOneTableDetails(const char *schemaname,
 	for (i = 0; i < numrows; i++)
 	{
 		/* Name */
+#ifdef WIN32
+		cells[i * cols + 0] = mbvalidate(PQgetvalue(res, i, 0), myopt.encoding);
+#else
 		cells[i * cols + 0] = PQgetvalue(res, i, 0);	/* don't free this
 														 * afterwards */
+#endif
+
 		/* Type */
+#ifdef WIN32
+		cells[i * cols + 1] = mbvalidate(PQgetvalue(res, i, 1), myopt.encoding);
+#else
 		cells[i * cols + 1] = PQgetvalue(res, i, 1);	/* don't free this
 														 * either */
+#endif
 
 		/* Extra: not null and default */
-		/* (I'm cutting off the 'default' string at 128) */
 		if (show_modifiers)
 		{
 			resetPQExpBuffer(&tmpbuf);
-			if (strcmp(PQgetvalue(res, i, 2), "t") == 0)
+			if (strcmp(PQgetvalue(res, i, 3), "t") == 0)
 				appendPQExpBufferStr(&tmpbuf, "not null");
 
 			/* handle "default" here */
-			if (strcmp(PQgetvalue(res, i, 3), "t") == 0)
+			/* (note: above we cut off the 'default' string at 128) */
+			if (strlen(PQgetvalue(res, i, 2)) != 0)
 			{
-				PGresult   *result;
-
-				printfPQExpBuffer(&buf,
-								  "SELECT substring(d.adsrc for 128) FROM pg_catalog.pg_attrdef d\n"
-								  "WHERE d.adrelid = '%s' AND d.adnum = %s",
-								  oid, PQgetvalue(res, i, 4));
-
-				result = PSQLexec(buf.data, false);
-
 				if (tmpbuf.len > 0)
 					appendPQExpBufferStr(&tmpbuf, " ");
-
 				appendPQExpBuffer(&tmpbuf, "default %s",
-								  result ? PQgetvalue(result, 0, 0) : "?");
-
-				PQclear(result);
+								  PQgetvalue(res, i, 2));
 			}
 
+#ifdef WIN32
+			cells[i * cols + 2] = xstrdup(mbvalidate(tmpbuf.data, myopt.encoding));
+#else
 			cells[i * cols + 2] = xstrdup(tmpbuf.data);
+#endif
 		}
 
 		/* Description */
 		if (verbose)
+#ifdef WIN32
+			cells[i * cols + cols - 1] = mbvalidate(PQgetvalue(res, i, 5), myopt.encoding);
+#else
 			cells[i * cols + cols - 1] = PQgetvalue(res, i, 5);
+#endif
 	}
 
 	/* Make title */
@@ -912,7 +926,7 @@ describeOneTableDetails(const char *schemaname,
 		footers = xmalloczero((rule_count + 2) * sizeof(*footers));
 		footers[count_footers] = xmalloc(64 + strlen(view_def));
 		snprintf(footers[count_footers], 64 + strlen(view_def),
-				 _("View definition: %s"), view_def);
+				 _("View definition:\n%s"), view_def);
 		count_footers++;
 
 		/* print rules */
@@ -941,12 +955,14 @@ describeOneTableDetails(const char *schemaname,
 				   *result2 = NULL,
 				   *result3 = NULL,
 				   *result4 = NULL,
-				   *result5 = NULL;
+				   *result5 = NULL,
+				   *result6 = NULL;
 		int			check_count = 0,
 					index_count = 0,
 					foreignkey_count = 0,
 					rule_count = 0,
-					trigger_count = 0;
+					trigger_count = 0,
+					inherits_count = 0;
 		int			count_footers = 0;
 
 		/* count indexes */
@@ -970,13 +986,18 @@ describeOneTableDetails(const char *schemaname,
 		if (tableinfo.checks)
 		{
 			printfPQExpBuffer(&buf,
-							  "SELECT consrc, conname\n"
+							  "SELECT "
+							  "pg_catalog.pg_get_constraintdef(r.oid, true), "
+							  "conname\n"
 							  "FROM pg_catalog.pg_constraint r\n"
-						   "WHERE r.conrelid = '%s' AND r.contype = 'c'",
+							  "WHERE r.conrelid = '%s' AND r.contype = 'c'",
 							  oid);
 			result2 = PSQLexec(buf.data, false);
 			if (!result2)
+			{
+				PQclear(result1);
 				goto error_return;
+			}
 			else
 				check_count = PQntuples(result2);
 		}
@@ -985,13 +1006,17 @@ describeOneTableDetails(const char *schemaname,
 		if (tableinfo.hasrules)
 		{
 			printfPQExpBuffer(&buf,
-							  "SELECT r.rulename\n"
+							  "SELECT r.rulename, trim(trailing ';' from pg_catalog.pg_get_ruledef(r.oid))\n"
 							  "FROM pg_catalog.pg_rewrite r\n"
 							  "WHERE r.ev_class = '%s'",
 							  oid);
 			result3 = PSQLexec(buf.data, false);
 			if (!result3)
+			{
+				PQclear(result1);
+				PQclear(result2);
 				goto error_return;
+			}
 			else
 				rule_count = PQntuples(result3);
 		}
@@ -1000,10 +1025,10 @@ describeOneTableDetails(const char *schemaname,
 		if (tableinfo.triggers)
 		{
 			printfPQExpBuffer(&buf,
-							  "SELECT t.tgname\n"
+				 "SELECT t.tgname, pg_catalog.pg_get_triggerdef(t.oid)\n"
 							  "FROM pg_catalog.pg_trigger t\n"
 							  "WHERE t.tgrelid = '%s' "
-							  "and (not tgisconstraint "
+							  "AND (not tgisconstraint "
 							  " OR NOT EXISTS"
 							  "  (SELECT 1 FROM pg_catalog.pg_depend d "
 							  "   JOIN pg_catalog.pg_constraint c ON (d.refclassid = c.tableoid AND d.refobjid = c.oid) "
@@ -1011,7 +1036,12 @@ describeOneTableDetails(const char *schemaname,
 							  oid);
 			result4 = PSQLexec(buf.data, false);
 			if (!result4)
+			{
+				PQclear(result1);
+				PQclear(result2);
+				PQclear(result3);
 				goto error_return;
+			}
 			else
 				trigger_count = PQntuples(result4);
 		}
@@ -1027,115 +1057,144 @@ describeOneTableDetails(const char *schemaname,
 							  oid);
 			result5 = PSQLexec(buf.data, false);
 			if (!result5)
+			{
+				PQclear(result1);
+				PQclear(result2);
+				PQclear(result3);
+				PQclear(result4);
 				goto error_return;
+			}
 			else
 				foreignkey_count = PQntuples(result5);
 		}
 
-		footers = xmalloczero((index_count + check_count + rule_count + trigger_count + foreignkey_count + 1)
+		/* count inherited tables */
+		printfPQExpBuffer(&buf, "SELECT c.relname FROM pg_catalog.pg_class c, pg_catalog.pg_inherits i WHERE c.oid=i.inhparent AND i.inhrelid = '%s' ORDER BY inhseqno ASC", oid);
+
+		result6 = PSQLexec(buf.data, false);
+		if (!result6)
+			goto error_return;
+		else
+			inherits_count = PQntuples(result6);
+
+		footers = xmalloczero((index_count + check_count + rule_count + trigger_count + foreignkey_count + inherits_count + 6)
 							  * sizeof(*footers));
 
 		/* print indexes */
-		for (i = 0; i < index_count; i++)
+		if (index_count > 0)
 		{
-			char	   *s = _("Indexes");
-			const char *indexdef;
-			const char *usingpos;
-
-			if (i == 0)
-				printfPQExpBuffer(&buf, "%s: %s", s,
-								  PQgetvalue(result1, i, 0));
-			else
-				printfPQExpBuffer(&buf, "%*s  %s", (int) strlen(s), "",
-								  PQgetvalue(result1, i, 0));
-
-			/* Label as primary key or unique (but not both) */
-			appendPQExpBuffer(&buf,
-							  strcmp(PQgetvalue(result1, i, 1), "t") == 0
-							  ? _(" primary key") :
-							  (strcmp(PQgetvalue(result1, i, 2), "t") == 0
-							   ? _(" unique")
-							   : ""));
-
-			/* Everything after "USING" is echoed verbatim */
-			indexdef = PQgetvalue(result1, i, 3);
-			usingpos = strstr(indexdef, " USING ");
-			if (usingpos)
-				indexdef = usingpos + 7;
-
-			appendPQExpBuffer(&buf, " %s", indexdef);
-
-			if (i < index_count - 1)
-				appendPQExpBuffer(&buf, ",");
-
+			printfPQExpBuffer(&buf, _("Indexes:"));
 			footers[count_footers++] = xstrdup(buf.data);
+			for (i = 0; i < index_count; i++)
+			{
+				const char *indexdef;
+				const char *usingpos;
+
+				/* Output index name */
+				printfPQExpBuffer(&buf, _("    \"%s\""),
+								  PQgetvalue(result1, i, 0));
+
+				/* Label as primary key or unique (but not both) */
+				appendPQExpBuffer(&buf,
+							  strcmp(PQgetvalue(result1, i, 1), "t") == 0
+								  ? _(" primary key,") :
+							 (strcmp(PQgetvalue(result1, i, 2), "t") == 0
+							  ? _(" unique,")
+							  : ""));
+
+				/* Everything after "USING" is echoed verbatim */
+				indexdef = PQgetvalue(result1, i, 3);
+				usingpos = strstr(indexdef, " USING ");
+				if (usingpos)
+					indexdef = usingpos + 7;
+
+				appendPQExpBuffer(&buf, " %s", indexdef);
+
+				footers[count_footers++] = xstrdup(buf.data);
+			}
 		}
 
-
 		/* print check constraints */
-		for (i = 0; i < check_count; i++)
+		if (check_count > 0)
 		{
-			char	   *s = _("Check constraints");
-
-			if (i == 0)
-				printfPQExpBuffer(&buf, _("%s: \"%s\" %s"),
-								  s,
-								  PQgetvalue(result2, i, 1),
-								  PQgetvalue(result2, i, 0));
-			else
-				printfPQExpBuffer(&buf, _("%*s  \"%s\" %s"),
-								  (int) strlen(s), "",
-								  PQgetvalue(result2, i, 1),
-								  PQgetvalue(result2, i, 0));
+			printfPQExpBuffer(&buf, _("Check constraints:"));
 			footers[count_footers++] = xstrdup(buf.data);
+			for (i = 0; i < check_count; i++)
+			{
+				printfPQExpBuffer(&buf, _("    \"%s\" %s"),
+								  PQgetvalue(result2, i, 1),
+								  PQgetvalue(result2, i, 0));
+
+				footers[count_footers++] = xstrdup(buf.data);
+			}
 		}
 
 		/* print foreign key constraints */
-		for (i = 0; i < foreignkey_count; i++)
+		if (foreignkey_count > 0)
 		{
-			char	   *s = _("Foreign Key constraints");
-
-			if (i == 0)
-				printfPQExpBuffer(&buf, _("%s: %s %s"),
-								  s,
-								  PQgetvalue(result5, i, 0),
-								  PQgetvalue(result5, i, 1));
-			else
-				printfPQExpBuffer(&buf, _("%*s  %s %s"),
-								  (int) strlen(s), "",
-								  PQgetvalue(result5, i, 0),
-								  PQgetvalue(result5, i, 1));
-			if (i < foreignkey_count - 1)
-				appendPQExpBuffer(&buf, ",");
-
+			printfPQExpBuffer(&buf, _("Foreign-key constraints:"));
 			footers[count_footers++] = xstrdup(buf.data);
+			for (i = 0; i < foreignkey_count; i++)
+			{
+				printfPQExpBuffer(&buf, _("    \"%s\" %s"),
+								  PQgetvalue(result5, i, 0),
+								  PQgetvalue(result5, i, 1));
+
+				footers[count_footers++] = xstrdup(buf.data);
+			}
 		}
 
 		/* print rules */
-		for (i = 0; i < rule_count; i++)
+		if (rule_count > 0)
 		{
-			char	   *s = _("Rules");
-
-			if (i == 0)
-				printfPQExpBuffer(&buf, "%s: %s", s, PQgetvalue(result3, i, 0));
-			else
-				printfPQExpBuffer(&buf, "%*s  %s", (int) strlen(s), "", PQgetvalue(result3, i, 0));
-			if (i < rule_count - 1)
-				appendPQExpBuffer(&buf, ",");
-
+			printfPQExpBuffer(&buf, _("Rules:"));
 			footers[count_footers++] = xstrdup(buf.data);
+			for (i = 0; i < rule_count; i++)
+			{
+				const char *ruledef;
+
+				/* Everything after "CREATE RULE" is echoed verbatim */
+				ruledef = PQgetvalue(result3, i, 1);
+				ruledef += 12;
+
+				printfPQExpBuffer(&buf, "    %s", ruledef);
+
+				footers[count_footers++] = xstrdup(buf.data);
+			}
 		}
 
 		/* print triggers */
-		for (i = 0; i < trigger_count; i++)
+		if (trigger_count > 0)
 		{
-			char	   *s = _("Triggers");
+			printfPQExpBuffer(&buf, _("Triggers:"));
+			footers[count_footers++] = xstrdup(buf.data);
+			for (i = 0; i < trigger_count; i++)
+			{
+				const char *tgdef;
+				const char *usingpos;
+
+				/* Everything after "TRIGGER" is echoed verbatim */
+				tgdef = PQgetvalue(result4, i, 1);
+				usingpos = strstr(tgdef, " TRIGGER ");
+				if (usingpos)
+					tgdef = usingpos + 9;
+
+				printfPQExpBuffer(&buf, "    %s", tgdef);
+
+				footers[count_footers++] = xstrdup(buf.data);
+			}
+		}
+
+		/* print inherits */
+		for (i = 0; i < inherits_count; i++)
+		{
+			char	   *s = _("Inherits");
 
 			if (i == 0)
-				printfPQExpBuffer(&buf, "%s: %s", s, PQgetvalue(result4, i, 0));
+				printfPQExpBuffer(&buf, "%s: %s", s, PQgetvalue(result6, i, 0));
 			else
-				printfPQExpBuffer(&buf, "%*s  %s", (int) strlen(s), "", PQgetvalue(result4, i, 0));
-			if (i < trigger_count - 1)
+				printfPQExpBuffer(&buf, "%*s  %s", (int) strlen(s), "", PQgetvalue(result6, i, 0));
+			if (i < inherits_count - 1)
 				appendPQExpBuffer(&buf, ",");
 
 			footers[count_footers++] = xstrdup(buf.data);
@@ -1149,6 +1208,7 @@ describeOneTableDetails(const char *schemaname,
 		PQclear(result3);
 		PQclear(result4);
 		PQclear(result5);
+		PQclear(result6);
 	}
 
 	printTable(title.data, headers,
@@ -1315,9 +1375,9 @@ listTables(const char *tabtypes, const char *pattern, bool verbose)
 
 	/*
 	 * If showSystem is specified, show only system objects (those in
-	 * pg_catalog).  Otherwise, suppress system objects, including
-	 * those in pg_catalog and pg_toast.  (We don't want to hide temp
-	 * tables though.)
+	 * pg_catalog).  Otherwise, suppress system objects, including those
+	 * in pg_catalog and pg_toast.	(We don't want to hide temp tables
+	 * though.)
 	 */
 	if (showSystem)
 		appendPQExpBuffer(&buf, "      AND n.nspname = 'pg_catalog'\n");
@@ -1406,6 +1466,149 @@ listDomains(const char *pattern)
 	return true;
 }
 
+/*
+ * \dc
+ *
+ * Describes conversions.
+ */
+bool
+listConversions(const char *pattern)
+{
+	PQExpBufferData buf;
+	PGresult   *res;
+	printQueryOpt myopt = pset.popt;
+
+	initPQExpBuffer(&buf);
+
+	printfPQExpBuffer(&buf,
+					  "SELECT n.nspname AS \"%s\",\n"
+					  "       c.conname AS \"%s\",\n"
+	"       pg_catalog.pg_encoding_to_char(c.conforencoding) AS \"%s\",\n"
+	"       pg_catalog.pg_encoding_to_char(c.contoencoding) AS \"%s\",\n"
+					  "       CASE WHEN c.condefault THEN '%s'\n"
+					  "       ELSE '%s' END AS \"%s\"\n"
+		   "FROM pg_catalog.pg_conversion c, pg_catalog.pg_namespace n\n"
+					  "WHERE n.oid = c.connamespace\n",
+					  _("Schema"),
+					  _("Name"),
+					  _("Source"),
+					  _("Destination"),
+					  _("yes"),
+					  _("no"),
+					  _("Default?"));
+
+	processNamePattern(&buf, pattern, true, false,
+					   "n.nspname", "c.conname", NULL,
+					   "pg_catalog.pg_conversion_is_visible(c.oid)");
+
+	appendPQExpBuffer(&buf, "ORDER BY 1, 2;");
+
+	res = PSQLexec(buf.data, false);
+	termPQExpBuffer(&buf);
+	if (!res)
+		return false;
+
+	myopt.nullPrint = NULL;
+	myopt.title = _("List of conversions");
+
+	printQuery(res, &myopt, pset.queryFout);
+
+	PQclear(res);
+	return true;
+}
+
+/*
+ * \dC
+ *
+ * Describes casts.
+ */
+bool
+listCasts(const char *pattern)
+{
+	PQExpBufferData buf;
+	PGresult   *res;
+	printQueryOpt myopt = pset.popt;
+
+	initPQExpBuffer(&buf);
+/* NEED LEFT JOIN FOR BINARY CASTS */
+	printfPQExpBuffer(&buf,
+		   "SELECT pg_catalog.format_type(castsource, NULL) AS \"%s\",\n"
+		   "       pg_catalog.format_type(casttarget, NULL) AS \"%s\",\n"
+					  "       CASE WHEN castfunc = 0 THEN '%s'\n"
+					  "            ELSE p.proname\n"
+					  "       END as \"%s\",\n"
+					  "       CASE WHEN c.castcontext = 'e' THEN '%s'\n"
+					  "            WHEN c.castcontext = 'a' THEN '%s'\n"
+					  "            ELSE '%s'\n"
+					  "       END as \"%s\"\n"
+			 "FROM pg_catalog.pg_cast c LEFT JOIN pg_catalog.pg_proc p\n"
+					  "     ON c.castfunc = p.oid\n"
+					  "ORDER BY 1, 2",
+					  _("Source type"),
+					  _("Target type"),
+					  _("(binary compatible)"),
+					  _("Function"),
+					  _("no"),
+					  _("in assignment"),
+					  _("yes"),
+					  _("Implicit?"));
+
+	res = PSQLexec(buf.data, false);
+	termPQExpBuffer(&buf);
+	if (!res)
+		return false;
+
+	myopt.nullPrint = NULL;
+	myopt.title = _("List of casts");
+
+	printQuery(res, &myopt, pset.queryFout);
+
+	PQclear(res);
+	return true;
+}
+
+/*
+ * \dn
+ *
+ * Describes schemas (namespaces)
+ */
+bool
+listSchemas(const char *pattern)
+{
+	PQExpBufferData buf;
+	PGresult   *res;
+	printQueryOpt myopt = pset.popt;
+
+	initPQExpBuffer(&buf);
+	printfPQExpBuffer(&buf,
+					  "SELECT n.nspname AS \"%s\",\n"
+					  "       u.usename AS \"%s\"\n"
+		"FROM pg_catalog.pg_namespace n LEFT JOIN pg_catalog.pg_user u\n"
+					  "       ON n.nspowner=u.usesysid\n",
+					  _("Name"),
+					  _("Owner"));
+
+	processNamePattern(&buf, pattern, false, false,
+					   NULL, "n.nspname", NULL,
+					   NULL);
+
+	appendPQExpBuffer(&buf, "ORDER BY 1;");
+
+	res = PSQLexec(buf.data, false);
+	termPQExpBuffer(&buf);
+	if (!res)
+		return false;
+
+	myopt.nullPrint = NULL;
+	myopt.title = _("List of schemas");
+
+	printQuery(res, &myopt, pset.queryFout);
+
+	PQclear(res);
+	return true;
+}
+
+
 /*
  * processNamePattern
  *
